@@ -12,9 +12,7 @@ import torch.tensor as tensor
 from torch import Tensor as T
 
 import torchsynth.util as util
-from torchsynth.config import DEBUG
-from torchsynth.default import DEFAULT_BATCH_SIZE, EPS
-from torchsynth.globals import SynthGlobals
+from torchsynth.config import BATCH_SIZE_FOR_REPRODUCIBILITY, SynthConfig
 from torchsynth.parameter import ModuleParameter, ModuleParameterRange
 from torchsynth.signal import Signal
 
@@ -36,13 +34,13 @@ class SynthModule(nn.Module):
 
     def __init__(
         self,
-        synthglobals: SynthGlobals,
+        synthconfig: SynthConfig,
         device: Optional[torch.device] = None,
         **kwargs: Dict[str, T],
     ):
         """
         Args:
-            synthglobals (:obj:`SynthGlobals`): These are global settings
+            synthconfig (:obj:`SynthGlobals`): These are global settings
             shared across all modules in the same synth.
 
             device (:obj:`torch.device`):
@@ -56,9 +54,9 @@ class SynthModule(nn.Module):
         batch_size is the number of settings we are rendering at once.
         """
         nn.Module.__init__(self)
-        self.synthglobals = synthglobals
+        self.synthconfig = synthconfig
         self.device = device
-        self.synthglobals.to(device)
+        self.synthconfig.to(device)
         self.torchparameters: nn.ParameterDict = nn.ParameterDict()
         self.parameter_ranges = []
         # If this module needs a random seed, here it is
@@ -78,7 +76,7 @@ class SynthModule(nn.Module):
                     ModuleParameter(
                         value=None,
                         parameter_name=parameter_range.name,
-                        data=torch.rand((self.synthglobals.batch_size,), device=device),
+                        data=torch.rand((self.synthconfig.batch_size,), device=device),
                         parameter_range=parameter_range,
                     )
                     for parameter_range in self.parameter_ranges
@@ -93,22 +91,26 @@ class SynthModule(nn.Module):
 
     @property
     def batch_size(self) -> T:
-        assert self.synthglobals.batch_size.ndim == 0
-        return self.synthglobals.batch_size
+        assert self.synthconfig.batch_size.ndim == 0
+        return self.synthconfig.batch_size
 
     @property
     def sample_rate(self) -> T:
-        assert self.synthglobals.sample_rate.ndim == 0
-        return self.synthglobals.sample_rate
+        assert self.synthconfig.sample_rate.ndim == 0
+        return self.synthconfig.sample_rate
 
     @property
     def nyquist(self):
         return self.sample_rate / 2.0
 
+    @property
+    def eps(self) -> float:
+        return self.synthconfig.eps
+
     @property
     def buffer_size(self) -> T:
-        assert self.synthglobals.buffer_size.ndim == 0
-        return self.synthglobals.buffer_size
+        assert self.synthconfig.buffer_size.ndim == 0
+        return self.synthconfig.buffer_size
 
     def to_buffer_size(self, signal: Signal) -> Signal:
         return util.fix_length(signal, self.buffer_size)
@@ -215,7 +217,7 @@ class SynthModule(nn.Module):
         """
         This handles the device transfer tasks that are not managed by PyTorch.
         """
-        self.synthglobals.to(device)
+        self.synthconfig.to(device)
         self.device = device
         # Currently the parameters ranges themselves are not move to a different
         # device, but each ParameterRange object is device aware if it needs to be
@@ -240,13 +242,13 @@ class ControlRateModule(SynthModule):
 
     @property
     def control_rate(self) -> T:
-        assert self.synthglobals.control_rate.ndim == 0
-        return self.synthglobals.control_rate
+        assert self.synthconfig.control_rate.ndim == 0
+        return self.synthconfig.control_rate
 
     @property
     def control_buffer_size(self) -> T:
-        assert self.synthglobals.control_buffer_size.ndim == 0
-        return self.synthglobals.control_buffer_size
+        assert self.synthconfig.control_buffer_size.ndim == 0
+        return self.synthconfig.control_buffer_size
 
     def to_buffer_size(self, signal: Signal) -> Signal:
         return util.fix_length(signal, self.control_buffer_size)
@@ -297,11 +299,11 @@ class ADSR(ControlRateModule):
 
     def __init__(
         self,
-        synthglobals: SynthGlobals,
+        synthconfig: SynthConfig,
         device: Optional[torch.device] = None,
         **kwargs: Dict[str, T],
     ):
-        super().__init__(synthglobals, device=device, **kwargs)
+        super().__init__(synthconfig, device=device, **kwargs)
 
         # Create some values that will be automatically loaded on device
         self.register_buffer("zero", tensor(0.0, device=self.device))
@@ -332,7 +334,7 @@ class ADSR(ControlRateModule):
         behind the scenes to make the playing experience feel natural.
         """
 
-        if DEBUG:
+        if self.synthconfig.debug:
             assert note_on_duration.ndim == 1
             assert torch.all(note_on_duration > 0.0)
 
@@ -383,7 +385,7 @@ class ADSR(ControlRateModule):
         # Shape ramps.
         ramp = ramp - start
         ramp = torch.maximum(ramp, self.zero)
-        ramp = (ramp + EPS) / duration + EPS
+        ramp = (ramp + self.eps) / duration + self.eps
         ramp = torch.minimum(ramp, self.one)
 
         """
@@ -431,7 +433,7 @@ class VCO(SynthModule):
     stationary audio signal at its base pitch.
 
     Args:
-        synthglobals (SynthGlobals) : global args, see SynthModule
+        synthconfig (SynthConfig) : global args, see SynthModule
         phase (:obj:'T',optional) :   initial phase values
     """
 
@@ -485,7 +487,7 @@ class VCO(SynthModule):
 
         control_as_frequency = self.make_control_as_frequency(midi_f0, mod_signal)
 
-        if DEBUG:
+        if self.synthconfig.debug:
             assert (control_as_frequency >= 0).all() and (
                 control_as_frequency <= self.nyquist
             ).all()
@@ -529,7 +531,7 @@ class SineVCO(VCO):
 class TorchFmVCO(VCO):
     """
     # TODO Turn this into its own voice so we can be a bit smarter about aliasing
-    # See https://github.com/turian/torchsynth/issues/145
+    # See https://github.com/torchsynth/torchsynth/issues/145
     Frequency modulation VCO. Takes `mod_signal` as instantaneous frequency.
 
     Typical modulation is calculated in pitch-space (midi). For FM to work,
@@ -618,23 +620,41 @@ class Noise(SynthModule):
     these in a synth then choose different seeds for each instance.
     """
 
-    def __init__(self, synthglobals: SynthGlobals, seed: int, **kwargs):
-        super().__init__(synthglobals, **kwargs)
+    # Do we really want deterministic noise within each batch?
+    # https://github.com/torchsynth/torchsynth/issues/250
+    noise_batch_size: int = BATCH_SIZE_FOR_REPRODUCIBILITY
+    # Unfortunately, Final is not supported until Python 3.8
+    # noise_batch_size: Final[int] = BATCH_SIZE_FOR_REPRODUCIBILITY
+
+    def __init__(self, synthconfig: SynthConfig, seed: int, **kwargs):
+        super().__init__(synthconfig, **kwargs)
+
+        # https://github.com/torchsynth/torchsynth/issues/255
+        if (
+            self.batch_size != BATCH_SIZE_FOR_REPRODUCIBILITY
+            and self.synthconfig.reproducible
+        ):
+            raise ValueError(
+                "Reproducibility currently only supported "
+                f"with batch_size = {BATCH_SIZE_FOR_REPRODUCIBILITY}. "
+                "If you want a different batch_size, "
+                "initialize SynthConfig with reproducible=False"
+            )
 
         # Pre-compute default batch size number of noise samples
         generator = torch.Generator(device="cpu").manual_seed(seed)
-        noise = torch.empty((DEFAULT_BATCH_SIZE, self.buffer_size), device="cpu")
+        noise = torch.empty((self.noise_batch_size, self.buffer_size), device="cpu")
         noise.data.uniform_(-1.0, 1.0, generator=generator)
 
         # Repeat the noise batches if the current batch size is larger than the default
-        if self.batch_size > DEFAULT_BATCH_SIZE:
-            if self.batch_size % DEFAULT_BATCH_SIZE != 0:
+        if self.batch_size > self.noise_batch_size:
+            if self.batch_size % self.noise_batch_size != 0:
                 raise ValueError(
                     "Batch size is not divisible by the default "
-                    f"batch size of {DEFAULT_BATCH_SIZE}"
+                    f"batch size of {self.noise_batch_size}"
                 )
 
-            noise = noise.repeat(self.batch_size // DEFAULT_BATCH_SIZE, 1)
+            noise = noise.repeat(self.batch_size // self.noise_batch_size, 1)
 
         self.register_buffer("noise", noise.to(self.device))
         self.offset = 0
@@ -642,20 +662,21 @@ class Noise(SynthModule):
     def _forward(self) -> Signal:
         # Return noise quickly if we are returning the whole batch
         # and don't need to offset at all.
-        if self.batch_size >= DEFAULT_BATCH_SIZE and self.offset == 0:
+        # TODO: Is this actually correct?
+        if self.batch_size >= self.noise_batch_size and self.offset == 0:
             return self.noise.as_subclass(Signal)
 
         # If the batch size is smaller than the default batch size then
         # we need to select a portion of the noise samples. Offset
         # is used to cycle through all the noise in the default batch size
         # or select the correct noise sample from the default batch
-        # TODO https://github.com/turian/torchsynth/issues/255
+        # TODO https://github.com/torchsynth/torchsynth/issues/255
         if self.offset == 0:
             noise = self.noise[: self.batch_size].as_subclass(Signal)
         else:
             noise = torch.roll(self.noise, (-self.offset, 0), dims=(0, 1))
 
-        self.offset = (self.offset + self.batch_size) % DEFAULT_BATCH_SIZE
+        self.offset = (self.offset + self.batch_size) % self.noise_batch_size
         return noise[: self.batch_size].as_subclass(Signal)
 
 
@@ -690,7 +711,7 @@ class LFO(ControlRateModule):
 
     def __init__(
         self,
-        synthglobals: SynthGlobals,
+        synthconfig: SynthConfig,
         exponent: T = tensor(2.718281828),  # e
         **kwargs: Dict[str, T],
     ):
@@ -705,7 +726,7 @@ class LFO(ControlRateModule):
                     description=f"Selection parameter for {lfo} LFO",
                 )
             )
-        super().__init__(synthglobals, **kwargs)
+        super().__init__(synthconfig, **kwargs)
         self.exponent = exponent
 
     def _forward(self, mod_signal: Signal) -> Signal:
@@ -763,7 +784,7 @@ class ModulationMixer(SynthModule):
 
     Parameters
     ----------
-    synthglobals (SynthGlobals) :   Synth config settings
+    synthconfig (SynthGlobals) :   Synth config settings
     n_input     (int)           :   Number of signals this module will mix
     n_output    (int)           :   Number of output signals
     curves  (optional, list)    :   A list of curve values to use for underlying
@@ -773,7 +794,7 @@ class ModulationMixer(SynthModule):
 
     def __init__(
         self,
-        synthglobals: SynthGlobals,
+        synthconfig: SynthConfig,
         n_input: int,
         n_output: int,
         curves: Optional[List[float]] = None,
@@ -800,7 +821,7 @@ class ModulationMixer(SynthModule):
                     )
                 )
 
-        super().__init__(synthglobals, **kwargs)
+        super().__init__(synthconfig, **kwargs)
         self.n_input = n_input
         self.n_output = n_output
 
@@ -829,7 +850,7 @@ class AudioMixer(SynthModule):
 
     def __init__(
         self,
-        synthglobals: SynthGlobals,
+        synthconfig: SynthConfig,
         n_input: int,
         curves: Optional[List[float]] = None,
         **kwargs: Dict[str, T],
@@ -854,7 +875,7 @@ class AudioMixer(SynthModule):
                 )
             )
 
-        super().__init__(synthglobals, **kwargs)
+        super().__init__(synthconfig, **kwargs)
         self.n_input = n_input
 
     def _forward(self, *signals: Signal) -> Signal:
@@ -876,18 +897,18 @@ class ControlRateUpsample(SynthModule):
     Upsample control signals to the global sampling rate
 
     Uses linear interpolation to resample an input control signal to the
-    audio buffer size set in synthglobals.
+    audio buffer size set in synthconfig.
     """
 
     def __init__(
         self,
-        synthglobals: SynthGlobals,
+        synthconfig: SynthConfig,
         device: Optional[torch.device] = None,
         **kwargs: Dict[str, T],
     ):
-        super().__init__(synthglobals, device, **kwargs)
+        super().__init__(synthconfig, device, **kwargs)
         self.upsample = torch.nn.Upsample(
-            self.synthglobals.buffer_size, mode="linear", align_corners=True
+            self.synthconfig.buffer_size, mode="linear", align_corners=True
         )
 
     def _forward(self, signal: Signal) -> Signal:
@@ -942,12 +963,12 @@ class SoftModeSelector(SynthModule):
     If there are n different modes, return a probability distribution over them.
 
     TODO: Would be nice to sample in a way that maximizes
-    KL-divergence from uniform: https://github.com/turian/torchsynth/issues/165
+    KL-divergence from uniform: https://github.com/torchsynth/torchsynth/issues/165
     """
 
     def __init__(
         self,
-        synthglobals: SynthGlobals,
+        synthconfig: SynthConfig,
         n_modes: int,
         exponent: T = tensor(2.718281828),  # e
         **kwargs: Dict[str, T],
@@ -955,7 +976,7 @@ class SoftModeSelector(SynthModule):
         """
         exponent determines how strongly to scale each [0,1] value prior
         to normalization. We should probably tune this:
-        https://github.com/turian/torchsynth/issues/165
+        https://github.com/torchsynth/torchsynth/issues/165
         """
         # Need to create the parameter ranges before calling super().__init
         self.default_parameter_ranges = [
@@ -967,7 +988,7 @@ class SoftModeSelector(SynthModule):
             )
             for i in range(n_modes)
         ]
-        super().__init__(synthglobals=synthglobals, **kwargs)
+        super().__init__(synthconfig=synthconfig, **kwargs)
         self.exponent = exponent
 
     def forward(self) -> Tuple[T, T]:
@@ -989,7 +1010,7 @@ class HardModeSelector(SynthModule):
 
     def __init__(
         self,
-        synthglobals: SynthGlobals,
+        synthconfig: SynthConfig,
         n_modes: int,
         **kwargs: Dict[str, T],
     ):
@@ -1003,7 +1024,7 @@ class HardModeSelector(SynthModule):
             )
             for i in range(n_modes)
         ]
-        super().__init__(synthglobals=synthglobals, **kwargs)
+        super().__init__(synthconfig=synthconfig, **kwargs)
 
     def forward(self) -> Tuple[T, T]:
         # Is this tensor creation slow?
